import time
import uuid
import typing
import asyncio

import redpesk_utils.log
import redpesk_utils.error
import redpesk_utils.common


class Request:
    def __init__(
            self,
            ticket: str = str(),
            tic: float = float()) -> None:
        self._tic = tic
        self._ticket = ticket if ticket else str(uuid.uuid4())

    @property
    def ticket(self) -> str:
        return self._ticket

    @property
    def tic(self) -> float:
        return self._tic


async def retry(request: Request, fail: bool, func: typing.Callable, *args: typing.Tuple[typing.Any]) -> typing.Any:
    res: typing.Any = None
    for idx in range(redpesk_utils.common.RETRY, 0, -1):
        err = str()
        try:
            res = await func(*args)
        except BaseException as error:
            err = str(error)
        finally:
            if fail and err or not fail and not err:
                return res
            else:
                redpesk_utils.log.red_log.warning(
                    redpesk_utils.log.format(
                        request.ticket, idx,
                        err if err else retry.__name__.upper()))

                await asyncio.sleep(redpesk_utils.common.DELAY)
    raise redpesk_utils.error.TimeoutError(
        redpesk_utils.common.FIELD_TIMEOUT)  # pragma: no cover


def report(tic: float) -> typing.Tuple[str]:
    return (f"{(time.perf_counter() - tic):0.4f}s",)
